unit Thd_Ini;

// Copyright  2001 by Ziff Davis Media, Inc.
// Written by Neil J. Rubenking

interface
uses
  Windows, SysUtils, Classes, CoaThds;

type
  TSearchINIThread = class(TCOAFileThread)
  protected
    procedure SearchFile(const S : String; Atts : DWORD); override;
  END;

  TChangeIniObj = class(TChangeObj)
  protected
    function GetHint : String; override;
  public
    function Undo :  Boolean; override;
    procedure ReportText(VAR T : TextFile; VAR MainS : String);
      override;
    procedure ReportHTML(VAR T : TextFile; VAR MainS : String);
      override;
  end;

implementation
uses CoaGenl, coafuncs;

function statString(stat : Integer) : String;
begin
  CASE stat AND $F OF
    statBlank : Result := '';
    statCheck : Result := '';
    statUndoX : begin
      Result := 'Status: Undo attempt failed';
      CASE stat AND (NOT $F) OF
        statFaildSavL : Result :=
          Result + '; could not save changed INI file to disk';
        statUndoxSect : Result :=
          Result + '; could not find changed section';
        statUndoxKeyK : Result :=
          Result + '; could not find changed key';
        statUndoxKeyS : Result :=
          Result + '; could not find section for changed key';
        statUndoxDatK : Result :=
          Result + '; could not find changed data';
        statUndoxDatS : Result :=
          Result + '; could not find section for changed data';
      end;
    end;
    statUndOK : Result := 'Status: Undo successful';
    statFaild : CASE stat AND (NOT $F) OF
      statFaildSavL : Result :=
        'Status: Change attempt failed; '+
          'could not save changed INI file to disk';
      ELSE Result := 'Status: Change attempt failed';
    end;
  END;
end;

procedure TSearchINIThread.SearchFile(const S : String; Atts : DWORD);
VAR
  TS    : TStringList;
  varS  : String;
  keyS  : String;
  datS  : String;
  N, P  : Integer;
  short : Boolean;
  slash : Boolean;
begin
  fHitStart := fHits; // Record the number of hits before we start,
                      // in case we need to fail-back
  fCurItem  := S;
  TS        := TStringList.Create;
  try try
    TS.LoadFromFile(fCurPath+'\'+S);
    fSub1 := '<no section>'; // fSub1 will hold the current section
    fSub2 := '';             // fSub2 will hold the key name
    fData1 := -1;             // will hold INI key line-number
    fData2 := DWORD(-1);      // will hold INI section line-number
    FOR N := 0 TO TS.Count-1 DO
      begin
        IF Terminated THEN Break;
        fData1 := N;         // Save line number for fast undo
        varS   := Trim(TS[N]);
        IF Length(varS) > 0 THEN
         CASE varS[1] OF
           ';' : ; // ignore comment
           '[' : begin // Starts with bracket - a header?
             P := Pos(']', varS);
             IF P > 0 THEN
               begin
                 fChgType := chgIniSection;
                 fSub1    := Copy(varS, 2, P-2);
                 fData2   := N;  // Save line number for fast undo
                 IF DoFind(fSub1, short, slash) THEN
                   begin
                     fOld := fSub1;
                     fNew := StrChangeSmart(fOld, fFindLong,
                       fReplLong, fFindShrt, fReplShrt, short,slash);
                     TS[N] := '['+fNew+']';
                     fSub1 := fNew;
                     Synchronize(SyncAddItem);
                   end;
               end;
           end;
           ELSE
             begin
               P := Pos('=', varS);
               IF P > 0 THEN
                 begin
                   keyS := varS; SetLength(KeyS, P-1);
                   datS := varS; system.Delete(datS, 1, P);
                   fSub2 := keyS;
                   IF DoFind(KeyS, short, slash) THEN
                     begin
                       fChgType := chgIniKey;
                       fOld := KeyS;
                       fNew := StrChangeSmart(fOld, fFindLong,
                         fReplLong, fFindShrt, fReplShrt, short,
                         slash);
                       KeyS := fNew;
                       TS[N] := KeyS + '=' + datS;
                       Synchronize(SyncAddItem);
                     end;
                   IF DoFind(datS, short, slash) THEN
                     begin
                       fChgType := chgIniData;
                       fOld := datS;
                       fNew := StrChangeSmart(fOld, fFindLong,
                         fReplLong, fFindShrt, fReplShrt, short,
                         slash);
                       datS := fNew;
                       TS[N] := KeyS + '=' + datS;
                       Synchronize(SyncAddItem);
                     end;
                 end;
             end;
         END;
      end;
    // IF there were any hits, write the file back
    IF fHits > fHitStart THEN
      begin
        SetFileAttributes(PChar(S), 0);
        IF DeleteFile(fCurPath+'\'+S) THEN
           TS.SaveToFile(fCurPath+'\'+S)
        ELSE
          begin
            // If we couldn't write the file, retroactively fail
            // all the reported hits
            fStatus := statFaild OR statFaildSavI;
            Synchronize(SyncFailBack);
          end;
        SetFileAttributes(PChar(S), Atts);
      end;
  finally
    TS.Free;
  end;
  except
    ON EInOutError DO;
  end;
end;

{ TChangeIniObj }

function TChangeIniObj.GetHint: String;
begin
  Result := fLoc+'\'+fItm+#13#10+'  *  *  *'+#13#10;
  CASE fTyp OF
    chgIniSection : Result := Result + 'Changed: Section';
    chgIniKey     : Result := Result +
      Format('Changed: Key name in section [%s]', [fSub1]);
    chgIniData    : Result := Result +
      Format('Changed: Data for key "%s" in section [%s]',
      [fSub2, fSub1]);
  END;
  CASE fUndo AND $F OF
    statBlank : ;
    statCheck : ;
    ELSE Result := Result + #13#10 + statString(fUndo);
  END;
end;

procedure TChangeIniObj.ReportHTML(var T: TextFile;
  VAR MainS: String);
begin
  IF MainS <> fLoc+'\'+fItm THEN
    begin
      MainS := fLoc+'\'+fItm;
      WriteLn(T, '<H3>', MainS, '</H3>');
    end;
  CASE fTyp OF
    chgIniSection : WriteLn(T, 'Changed: Section<BR>');
    chgIniKey     : WriteLn(T,
      Format('Changed: Key name in section [%s]<BR>', [fSub1]));
    chgIniData    : WriteLn(T,
      Format('Changed: Data for key "%s" in section [%s]<BR>',
      [fSub2, fSub1]));
  END;
  WriteLn(T, b,b,'Old: ', fOld, '<BR>');
  WriteLn(T, b,b,'New: ', fNew, '<BR>');
  CASE fUndo AND $F OF
    statBlank : ;
    statCheck : ;
    ELSE WriteLn(T, b,b,statString(fUndo), '<BR>');
  END;
end;

procedure TChangeIniObj.ReportText(var T: TextFile;
  VAR MainS : String);
begin
  IF MainS <> fLoc+'\'+fItm THEN
    begin
      MainS := fLoc+'\'+fItm;
      WriteLn(T);
      WriteLn(T, MainS);
    end;
  CASE fTyp OF
    chgIniSection : WriteLn(T, #9'Changed: Section');
    chgIniKey     : WriteLn(T,
      Format(#9'Changed: Key name in section [%s]', [fSub1]));
    chgIniData    : WriteLn(T,
      Format(#9'Changed: Data for key "%s" in section [%s]',
      [fSub2, fSub1]));
  END;
  WriteLn(T, #9#9'Old:'#9, fOld);
  WriteLn(T, #9#9'New:'#9, fNew);
  CASE fUndo AND $F OF
    statBlank : ;
    statCheck : ;
    ELSE WriteLn(T, #9#9, statString(fUndo));
  END;
end;

function TChangeIniObj.Undo: Boolean;
VAR
  TS     : TStringList;
  VarS   : String;
  KeyS   : String;
  DatS   : String;
  N, P   : Integer;
  GotSec : Boolean;
  Atts   : DWORD;
begin
  fUndo := statUndOX;
  TS    := TStringList.Create;
  try
    TS.LoadFromFile(fLoc+'\'+fItm);
    IF Integer(fData2) >= 0 THEN
      VarS   := Trim(TS[fData2])
    ELSE
      VarS := '';
    P      := Pos(']', varS);
    GotSec := ((P > 0) AND (Copy(varS, 2, P-2) = fSub1))
      OR ('<no section>' = fSub1);
    IF GotSec THEN
      begin
        // We found the remembered section at the correct line
        VarS := Trim(TS[fData1]);
        P := Pos('=', varS);
        IF P > 0 THEN
          begin
            keyS := varS; SetLength(KeyS, P-1);
            datS := varS; system.Delete(datS, 1, P);
          end
        ELSE
          begin
            KeyS := '';
            DatS := '';
          end;
        // Is the remembered data at its correct line?
        CASE fTyp OF
          chgIniSection : IF fData1 = Integer(fData2) THEN
          begin
            fUndo := statUndOK;
            TS[fData1] := '['+fOld+']';
          end;
          chgIniKey     : IF KeyS = fNew THEN
          begin
            fUndo := statUndOK;
            TS[fData1] := fOld + '=' + DatS;
          end;
          chgIniData    : IF DatS = fNew THEN
          begin
            fUndo := statUndOK;
            TS[fData1] := KeyS + '=' + fOld;
          end;
        END;
      end;
    // If we didn't find the changed item at the specified
    // line, search the file for it. INI files are always
    // getting re-ordered anyway.
    IF fUndo <> statUndOK THEN
      begin
        GotSec := fSub1='<no section>';
        N := 0;
        WHILE (NOT Gotsec) AND (N < TS.Count) DO
          begin
            VarS := Trim(TS[N]);
            IF (VarS <> '') AND (VarS[1] = '[') THEN
              begin
                P := Pos(']', VarS);
                IF Copy(varS, 2, P-2) = fSub1 THEN
                  begin
                    GotSec := True;
                    Break;
                  end;
              end;
            Inc(N);
          end;
        IF GotSec THEN
          begin
            CASE fTyp OF
              chgIniSection : begin
                fUndo := statUndOK;
                TS[N] := '['+fOld+']';
              end;
              chgIniKey     : begin
                Inc(N);
                WHILE N < TS.Count DO
                  begin
                    varS := Trim(TS[N]);
                    IF VarS <> '' THEN
                      begin
                        // If we hit the next section heading,
                        // the search has failed
                        IF VarS[1] = '[' THEN Break;
                        P := Pos('=', VarS);
                        IF P > 0 THEN
                          begin
                            keyS := varS;
                            SetLength(KeyS, P-1);
                            datS := varS;
                            system.Delete(datS, 1, P);
                            IF KeyS = fNew THEN
                              begin
                                fUndo := statUndOK;
                                TS[N] := fOld + '=' + DatS;
                                Break;
                              end;
                          end;
                      end;
                    Inc(N);
                  end;
                IF fUndo <> statUndOK THEN
                  fUndo := statUndox OR statUndoxKeyK;
              end;
              chgIniData    : begin
                Inc(N);
                WHILE N < TS.Count DO
                  begin
                    varS := Trim(TS[N]);
                    IF VarS <> '' THEN
                      begin
                        // If we hit the next section heading,
                        // the search has failed
                        IF VarS[1] = '[' THEN Break;
                        P := Pos('=', VarS);
                        IF P > 0 THEN
                          begin
                            keyS := varS;
                            SetLength(KeyS, P-1);
                            datS := varS;
                            system.Delete(datS, 1, P);
                            IF DatS = fNew THEN
                              begin
                                fUndo := statUndOK;
                                TS[N] := KeyS + '=' + fOld;
                                Break;
                              end;
                          end;
                      end;
                    Inc(N);
                  end;
                IF fUndo <> statUndOK THEN
                  fUndo := statUndox OR statUndoxDatK;
              end;
            END;
          end
        ELSE
          begin
            CASE fTyp OF
              chgIniSection : fUndo := statUndoX OR statUndoxSect;
              chgIniKey     : fUndo := statUndoX OR statUndoxKeyS;
              chgIniData    : fUndo := statUndoX OR statUndoxDatS;
            END;
          end;
      end;
    IF fUndo = statUndOK THEN
      begin
        Atts := GetFileAttributes(PChar(fLoc+'\'+fItm));
        SetFileAttributes(PChar(fLoc+'\'+fItm), 0);
        try
          TS.SaveToFile(fLoc+'\'+fItm);
        except
          ON Exception DO
            fUndo := statUndoX OR statFaildSavI;
        end;
        SetFileAttributes(PChar(fLoc+'\'+fItm), Atts);
      end;
    Result := fUndo = statUndOK;
  finally
    TS.Free;
  end;
end;

end.
