unit Thd_Lnk;

// Copyright  2001 by Ziff Davis Media, Inc.
// Written by Neil J. Rubenking

interface
uses
  Windows, SysUtils, Classes, CoaThds;

type
  TSearchLNKThread = class(TCOAFileThread)
  protected
    procedure SearchFile(const S : String; Atts : DWORD); override;
  public
    procedure Execute; override;
  END;

  TChangeLnkObj = class(TChangeObj)
  protected
    function GetHint : String; override;
  public
    function Undo : Boolean; override;
    procedure ReportText(VAR T : TextFile;
      VAR MainS : String); override;
    procedure ReportHTML(VAR T : TextFile;
      VAR MainS : String); override;
  end;

implementation
uses comobj, shlobj, ActiveX, CoaGenl, coafuncs;

function StatString(stat : Integer) : String;
begin
  CASE stat AND $F OF
    statBlank : Result := '';
    statCheck : Result := '';
    statUndoX : begin
      Result := 'Status: Undo attempt failed';
      CASE stat AND (NOT $F) OF
        statFaildArgs : Result := Result +
          '; could not change arguments';
        statFaildWork : Result := Result +
          '; could not change working directory';
        statFaildIcon : Result := Result +
          '; could not change icon location';
        statFaildPath : Result := Result +
          '; could not change shortcut''s target';
        statFaildSavL : Result := Result +
          '; could not save changed shortcut to disk';
      END;
    end;
    statUndOK : Result := 'Status: Undo successful';
    statFaild : begin
      Result := 'Status: Change attempt failed';
      CASE stat AND (NOT $F) OF
        statFaildArgs : Result := Result +
          '; could not change arguments';
        statFaildWork : Result := Result +
          '; could not change working directory';
        statFaildIcon : Result := Result +
          '; could not change icon location';
        statFaildPath : Result := Result +
          '; could not change shortcut''s target';
        statFaildSavL : Result := Result +
          '; could not save changed shortcut to disk';
      END;
    end;
  END;
end;

procedure TSearchLNKThread.Execute;
begin
  CoInitialize(nil);
  try
    inherited Execute;
  finally
    CoUninitialize;
  end;
end;

procedure TSearchLNKThread.SearchFile(const S : String; Atts: DWORD);
VAR
  fNameW     : ARRAY[0..MAX_PATH] OF WideChar;
  vUNK       : IUnknown;
  vISL       : IShellLink;
  vIPF       : IPersistFile;
  TempStr    : ARRAY[0..MAX_PATH] OF Char;
  short      : Boolean;
  slash      : Boolean;
  TW         : TWin32FindData;
  I          : Integer;
begin
  fCurItem := S;
  fData1 := 0;
  fHitStart := fHits;
  try
    StringToWideChar(fCurPath + '\' + S, fNameW, MAX_PATH);
    vUNK := CreateComObject(CLSID_ShellLink);
    vISL := vUNK AS IShellLink;
    vIPF := vUNK AS IPersistFile;
    try
      IF vIPF.Load(@fNameW, STGM_READWRITE) <> S_OK THEN Exit;
    except
      ON EOleSysError DO
        Exit;
    end;
    IF (vISL.GetArguments(TempStr, MAX_PATH) = S_OK) THEN
      begin
        IF DoFind(StrPas(TempStr), short, slash) THEN
          begin
            fOld      := TempStr;
            fNew      := StrChangeSmart(fOld, fFindLong, fReplLong,
              fFindShrt, fReplShrt, short, slash);
            StrLCopy(TempStr, PChar(fNew), MAX_PATH);
            fChgType  := chgLnkArgs;
            IF vISL.SetArguments(TempStr) = S_OK THEN
              fStatus := statBlank
            ELSE
              fStatus := statFaild OR statFaildArgs;
            Synchronize(SyncAddItem);
          end;
      end;
    IF (vISL.GetWorkingDirectory(TempStr, MAX_PATH) = S_OK) THEN
      begin
        IF DoFind(StrPas(TempStr), short, slash) THEN
          begin
            fOld      := TempStr;
            fNew      := StrChangeSmart(fOld, fFindLong, fReplLong,
              fFindShrt, fReplShrt, short, slash);
            StrLCopy(TempStr, PChar(fNew), MAX_PATH);
            fChgType  := chgLnkWorkDir;
            IF vISL.SetWorkingDirectory(TempStr) = S_OK THEN
              fStatus := statBlank
            ELSE
              fStatus := statFaild OR statFaildWork;
            Synchronize(SyncAddItem);
          end;
      end;
    IF (vISL.GetIconLocation(TempStr, MAX_PATH, I) = S_OK) THEN
      begin
        IF DoFind(StrPas(TempStr), short, slash) THEN
          begin
            fOld      := TempStr;
            fNew      := StrChangeSmart(fOld, fFindLong, fReplLong,
              fFindShrt, fReplShrt, short, slash);
            StrLCopy(TempStr, PChar(fNew), MAX_PATH);
            fChgType  := chgLnkIconLoc;
            fData1    := I;
            IF vISL.SetIconLocation(TempStr, I) = S_OK THEN
              fStatus := statBlank
            ELSE
              fStatus := statFaild OR statFaildIcon;
            Synchronize(SyncAddItem);
          end;
      end;
    IF (vISL.GetPath(TempStr, MAX_PATH, TW, 0) = S_OK) THEN
      begin
        IF DoFind(StrPas(TempStr), short, slash) THEN
          begin
            fOld      := TempStr;
            fNew      := StrChangeSmart(fOld, fFindLong, fReplLong,
              fFindShrt, fReplShrt, short, slash);
            StrLCopy(TempStr, PChar(fNew), MAX_PATH);
            fChgType  := chgLnkCommand;
            IF vISL.SetPath(TempStr) = S_OK THEN
              fStatus := statBlank
            ELSE
              fStatus := statFaild OR statFaildPath;
            Synchronize(SyncAddItem);
          end;
      end;
    // If there were any changes, try to write the file
    IF fHits > fHitStart THEN
      begin
        SetFileAttributes(PChar(fCurPath + '\' + S), 0);
        IF vIPF.Save(nil, False) <> S_OK THEN
          begin
            fStatus := statFaild OR statFaildSavL;
            Synchronize(SyncFailBack)
          end
        ELSE vIPF.SaveCompleted(nil);
        SetFileAttributes(PChar(fCurPath + '\' + S), Atts);
      end;
  except
    ON Exception DO;
  end;
end;

{ TChangeLnkObj }

function TChangeLnkObj.GetHint: String;
begin
  Result := fLoc+'\'+fItm+#13#10+'  *  *  *'+#13#10;
  CASE fTyp OF
    chgLnkCommand : Result := Result + 'Changed: Command';
    chgLnkIconLoc : Result := Result + 'Changed: Icon location';
    chgLnkWorkDir : Result := Result + 'Changed: Working directory';
    chgLnkArgs    : Result := Result + 'Changed: Arguments';
  END;
  CASE fUndo AND $F OF
    statBlank : ;
    statCheck : ;
    ELSE Result := Result + #13#10+statString(fUndo);
  END;
end;

procedure TChangeLnkObj.ReportHTML(var T: TextFile;
  VAR MainS: String);
begin
  IF MainS <> fLoc+'\'+fItm THEN
    begin
      MainS := fLoc+'\'+fItm;
      WriteLn(T, '<H3>', MainS, '</H3>');
    end;
  CASE fTyp OF
    chgLnkCommand   : WriteLn(T, 'Changed: Command<BR>');
    chgLnkIconLoc   : WriteLn(T, 'Changed: Icon location<BR>');
    chgLnkWorkDir   : WriteLn(T, 'Changed: Working directory<BR>');
    chgLnkArgs      : WriteLn(T, 'Changed: Arguments<BR>');
  END;
  WriteLn(T, b,b,'Old: ', fOld, '<BR>');
  WriteLn(T, b,b,'New: ', fNew, '<BR>');
  CASE fUndo AND $F OF
    statBlank : ;
    statCheck : ;
    ELSE WriteLn(T, b,b, statString(fUndo), '<BR>');
  END;
end;

procedure TChangeLnkObj.ReportText(var T: TextFile;
  VAR MainS : String);
begin
  IF MainS <> fLoc+'\'+fItm THEN
    begin
      MainS := fLoc+'\'+fItm;
      WriteLn(T);
      WriteLn(T, MainS);
    end;
  CASE fTyp OF
    chgLnkCommand   : WriteLn(T, #9'Changed: Command');
    chgLnkIconLoc   : WriteLn(T, #9'Changed: Icon location');
    chgLnkWorkDir   : WriteLn(T, #9'Changed: Working directory');
    chgLnkArgs      : WriteLn(T, #9'Changed: Arguments');
  END;
  WriteLn(T, #9#9'Old:'#9, fOld);
  WriteLn(T, #9#9'New:'#9, fNew);
  CASE fUndo AND $F OF
    statBlank : ;
    statCheck : ;
    ELSE WriteLn(T, #9#9, statString(fUndo));
  END;
end;

function TChangeLnkObj.Undo: Boolean;
VAR
  fNameW  : ARRAY[0..MAX_PATH] OF WideChar;
  vUNK    : IUnknown;
  vISL    : IShellLink;
  vIPF    : IPersistFile;
  TempStr : ARRAY[0..MAX_PATH] OF Char;
  Atts    : DWORD;
begin
  Result := False;
  fUndo  := statUndOK;
  try
    StringToWideChar(fLoc + '\' + fItm, fNameW, MAX_PATH);
    vUNK := CreateComObject(CLSID_ShellLink);
    vISL := vUNK AS IShellLink;
    vIPF := vUNK AS IPersistFile;
    try
      IF vIPF.Load(@fNameW, STGM_READWRITE) <> S_OK THEN Exit;
    except
      ON EOleSysError DO
        Exit;
    end;
    StrLCopy(TempStr, PChar(fOld), MAX_PATH);
    CASE fTyp OF
      chgLnkCommand   :
        IF vISL.SetPath(TempStr) <> S_OK THEN
          fUndo := statUndoX OR statFaildPath;
      chgLnkIconLoc   :
        IF vISL.SetIconLocation(TempStr, fData1) <> S_OK THEN
          fUndo := statUndoX OR statFaildIcon;
      chgLnkWorkDir   :
        IF vISL.SetWorkingDirectory(TempStr) <> S_OK THEN
          fUndo := statUndoX OR statFaildWork;
      chgLnkArgs      :
        IF vISL.SetArguments(TempStr) <> S_OK THEN
          fUndo := statUndoX OR statFaildArgs;
    end;
    IF fUndo = statUndOK THEN
      begin
        Atts := GetFileAttributes(PChar(fLoc + '\' + fItm));
        SetFileAttributes(PChar(fLoc + '\' + fItm), 0);
        IF vIPF.Save(nil, False) = S_OK THEN
          begin
            vIPF.SaveCompleted(nil);
            fUndo := statUndOK;
          end
        ELSE
          fUndo := statUndoX OR statFaildSavL;
        vIPF.SaveCompleted(nil);
        SetFileAttributes(PChar(fLoc + '\' + fItm), Atts);
      end;
    Result := fUndo = statUndOK;
  except
    ON Exception DO;
  end;
end;

end.
