unit coafuncs;

// Copyright  2001 by Ziff Davis Media, Inc.
// Written by Neil J. Rubenking

interface
USES Windows, SysUtils, Classes;

function GetFolder(H : HWnd; const S, Title : String;
  VAR GotS : String) : Boolean;
function GetShort(const S : String) : String;
function HasSlash(const S : String) : Boolean;
function SlashMatch(const S1, S2 : String) : Boolean;
function StrChangeSmart(CONST S, FindL, RepL, FindS, RepS : String;
  short, slash : Boolean) : String;

implementation
uses shlobj, ActiveX, FileCtrl, allfuncs;

VAR ShellVersion : DWORD;

function BrowseCallBackProc(hWindow : HWND; uMsg : Integer;
  lParam : LPARAM; lpData : Integer) : Integer; stdcall;
VAR buffer : ARRAY[0..MAX_PATH] OF Char;
begin
  Result := 0;
  IF uMsg = BFFM_INITIALIZED THEN
    begin
      StrCopy(Buffer, PChar(lpData));
      StrPCopy(Buffer, NoFinalSlash(StrPas(Buffer)));
      // Set the starting directory
      IF (StrLen(Buffer) = 2) AND (Buffer[1] = ':') THEN
        StrCat(Buffer, '\');
      SendMessage(hWindow, BFFM_SETSELECTION, 1, Integer(@Buffer));
    end
  ELSE IF uMsg = BFFM_SELCHANGED THEN
    begin
      SHGetPathFromIDList(pItemIDList(lParam), buffer);
      // Display the current selection
      SendMessage(hWindow, BFFM_SETSTATUSTEXT, 0, Integer(@Buffer));
    end
  ELSE
    begin
      StrPCopy(Buffer, Format('"%s" is not a valid folder name',
        [PChar(lParam)]));
      SendMessage(hWindow, BFFM_SETSTATUSTEXT, 0, Integer(@Buffer));
      MessageBeep(MB_ICONSTOP);
      Result := 1;
    end;
end;

function GetFolder(H : HWnd; const S, Title : String;
  VAR GotS : String) : Boolean;
const BIF_NEWDIALOGSTYLE = $40;  
VAR
  TB     : TBrowseInfo;
  IDL    : PItemIdList;
  buffer : ARRAY[0..MAX_PATH] OF Char;
begin
  FillChar(TB, SizeOf(TB), 0);
  WITH TB DO
    begin
      // Start browsing at My Computer - just show local drives
      SHGetSpecialFolderLocation(H, CSIDL_DRIVES, pIdlRoot);
      // Set the browse-dialog's title
      lpszTitle := PChar(Title);
      ulFlags   := BIF_RETURNONLYFSDIRS OR BIF_DONTGOBELOWDOMAIN OR
        BIF_STATUSTEXT;
      // If the shell supports it, include an edit box
      IF ShellVersion >= 471 THEN
        ulFlags := ulFlags OR BIF_EDITBOX OR BIF_VALIDATE;
      // If the shell supports it, use a new-style dialog
      IF ShellVersion >= 500 THEN
        ulFlags := ulFlags OR BIF_NEWDIALOGSTYLE;
      // Set the browse callback procedure
      lpfn      := @BrowseCallBackProc;
      // point the lParam at the desired starting folder
      StrPCopy(buffer, S);
      lParam    := Integer(@Buffer);
    end;
  IDL := ShBrowseForFolder(TB);
  try
    IF IDL <> nil THEN
      begin
        SHGetPathFromIDList(IDL, Buffer);
        CoTaskMemFree(IDL);
        GotS   := FinalSlash(StrPas(Buffer));
        Result := True;
      end
    ELSE Result := False;
  finally
    CoTaskMemFree(TB.pIdlRoot);
  end;
end;

function HasSlash(const S : String) : Boolean;
// True if S ends in a backslash
begin
  Result := (Length(S) > 0) AND (S[Length(S)] = '\');
end;

function SlashMatch(const S1, S2 : String) : Boolean;
// True if both S1 and S2 end in a backslash, or if neither does
begin
  Result := NOT (HasSlash(S1) XOR HasSlash(S2));
end;

function GetLong(const S : String) : String;
// Get a complete long filename for S
VAR slash : Boolean;
  function LongFor(const vDir, vName : String) : String;
  VAR
    FD : TWin32FindData;
    FH : THandle;
  begin
    FH := FindFirstFile(PChar(vDir), FD);
    IF FH = INVALID_HANDLE_VALUE THEN Result := vName
    ELSE
      begin
        Result := StrPas(FD.cFilename);
        Windows.FindClose(FH);
      end;
  end;

  function OneLong(const S : String) : String;
  VAR
    sDir, sName : String;
  begin
    sName := ExtractFileName(S);
    sDir := ExtractFileDir(S);
    IF sName = '' THEN
      Result := NoFinalSlash(sDir)
    ELSE IF sDir = '' THEN
      Result := NoFinalSlash(sName)
    ELSE
      Result := OneLong(sDir) + '\' + LongFor(S, sName);
  end;
begin
  Slash := HasSlash(S);
  Result := OneLong(NoFinalSlash(S));
  IF Slash THEN Result := FinalSlash(Result);
end;

function GetShort(const S : String) : String;
// Get a complete short pathname for S. Temporarily create path
// elements if necessary
VAR
  TS    : TStringList;
  N     : Integer;
  sDir  : String;
  sName : String;
  Slash : Boolean;
begin
  Result := S;
  IF S = '' THEN Exit;
  Slash := HasSlash(S);
  TS    := TStringList.Create;
  try
    sDir := Uppercase(S);
    REPEAT
      IF DirectoryExists(sDir) THEN Break;
      IF FileExists(sDir) THEN Break;
      TS.Add(sDir);
      sName := ExtractFileName(sDir);
      sDir  := ExtractFileDir(sDir);
    UNTIL (sName = '') OR (sDir = '');
    // TS now contains a list of full pathnames that don't exist,
    // with the longest first. We create them in reverse order,
    // building up to the longest.
    try
      FOR N := TS.Count-1 DOWNTO 0 DO
        MkDir(TS[N]);
      SetLength(Result, MAX_PATH);
      SetLength(Result, GetShortPathName(PChar(Uppercase(S)),
        PChar(Result), MAX_PATH));
      // Now we un-create them starting with the longest  
      FOR N := 0 TO TS.Count-1 DO
        RmDir(TS[N]);
      IF Slash THEN Result := FinalSlash(Result)
      ELSE Result := NoFinalSlash(Result);
    except
      On Exception DO
        Result := '';
    end;
  finally
    TS.Free;
  end;
end;

PROCEDURE StrChange(VAR S : String; CONST Find, Rep : String);
// Non-recursive procedure to change string "Find" to string "Rep"
// wherever found in string S. This is faster than the built-in
// StringReplace function, probably because the latter is more
// flexible and supports multibyte characterse
VAR Loc, First : Word;
BEGIN
  IF Length(Find) = 0 THEN Exit;
  IF Length(S) < Length(Find) THEN Exit;
  First := 1;
  WHILE True DO
    BEGIN
      Loc := Pos(Uppercase(Find),
                 Uppercase(Copy(S, First, Length(S)-First+1)));
      IF Loc = 0 THEN Exit;
      Loc := First + Loc - 1;
      System.Delete(S, Loc, length(Find));
      System.Insert(Rep, S, Loc);
      First := Loc + length(Rep);
    END;
END;

FUNCTION StrChangeF(CONST S, Find, Rep : String) : String;
begin
  Result := S;
  StrChange(Result, Find, Rep);
end;

function StrChangeSmart(CONST S, FindL, RepL, FindS, RepS : String;
  short, slash : Boolean) : String;
begin
  IF short THEN
    IF slash THEN
      begin // short, slash
        Result := NoFinalSlash(RepS);
      end
    ELSE
      begin // short, not slash
        Result := StrChangeF(S, FindS, RepS);
      end
  ELSE
    IF slash THEN
      begin // long, slash
        Result := NoFinalSlash(RepL);
      end
    ELSE
      begin // long, not slash
        Result := StrChangeF(S, FindL, RepL);
      end;
end;

function GetShellVersion : DWORD;
// 4.70 = 470, 5.0 = 500, etc. ANything before 4.70 = 0
type
  PDllVersionInfo = ^TDllVersionInfo;
  TDllVersionInfo = record
    cbSize          : DWORD;
    dwMajorVersion  : DWORD;
    dwMinorVersion  : DWORD;
    dwBuildNumber   : DWORD;
    dwPlatformID    : DWORD;
  end;
  DllGetVersionType = function (pdvi : PDllVersionInfo) :
    HRESULT; stdcall;
VAR
  LibH : hModule;
  DllGetVersion  : DllGetVersionType;
  DVI  : TDllVersionInfo;
begin
  Result := 0;
  LibH := LoadLibrary('Shell32.dll');
  IF LibH <= 32 THEN Exit;
  try
    @DllGetVersion := GetProcAddress(LibH, 'DllGetVersion');
    IF @DllGetVersion = nil THEN Exit;
    FillChar(DVI, SizeOf(DVI), 0);
    DVI.cbSize := SizeOf(DVI);
    IF DllGetVersion(@DVI) = NOERROR THEN
      begin
        Result := DVI.dwMajorVersion * 100;
        IF DVI.dwMinorVersion > 100 THEN Result := Result + 99
        ELSE Result := Result + DVI.dwMinorVersion;
      end;
  finally
    FreeLibrary(LibH);
  end;
end;

initialization
  ShellVersion := GetShellVersion;
  IF ShellVersion >= 500 THEN
    CoInitialize(nil);
finalization
  IF ShellVersion >= 500 THEN
    CoUninitialize;
end.
